#include "figarc.h"
#include <ostream>
#include <cmath>

using std::floor;
using std::ostream;


figarc::figarc(const double x1, const double y1, const double x2, const double y2, const double x3, const double y3) 
{
  KOM   = "(ARC)";	//kommentar
  TYP   =    5;		//(object_code) objekt Typ
  JUST  =    1;		//(sub_type)textausrichtung 0=links,1=mitte,2=rechts
  LINE_STYLE=0;		//(line_style)
  WIDTH =    1;		//(thickness) breite
  PEN_COLOR= 0;		//(pen_color)farbe:0=schwarz;1=blau;2=gruen;3=hellblau,4=rot,5=lila,6=gelb [0,31]
  FUELL_COLOR=7;		//(fill_color)farbe:0=schwarz;1=blau;2=gruen;3=hellblau,4=rot,5=lila,6=gelb [0,31]
  EBENE    =50;		//(depth)Ebene
  PINSL =   -1;		//(pen_style)
  FILL  =   -1;		//(area_fill)
  STYLE_VAL=.0;		//(style_val)
  JOIN_STYLE=0;		//(join_style)
  CAP_STYLE =0;		//(cap_style) Ecken 
  RADIUS   =-1;		//(radius)1/80 inch
  ARROW_F  = 0;		//(forward_arrow) Vorwaerts Pfeil 0=off 1=on
  ARROW_B  = 0;		//(backward_arrow) Rueckwaertspfeil 0=off 1=on
  POSX.resize(3);
  POSY.resize(3);
  POSX[0] = x1;
  POSY[0] = y1;
  POSX[1] = x2;
  POSY[1] = y2;
  POSX[2] = x3;
  POSY[2] = y3;

  direction = 0;	//(0: clockwise, 1: counterclockwise)

  double zaehler = ((x3-x2)*(x2-x1)-(y3-y2)*(y1-y2));
  double nenner = ((x3-x1)*(y1-y2)-(y1-y3)*(x2-x1));
  double k = zaehler / nenner;
  center_x = 0.5*(x1+x3+k*(y1-y3));
  center_y = 0.5*(y1+y3+k*(x3-x1));
  
  if ( (x1-center_x)*(y2-center_y)-(y1-center_y)*(x2-center_x) > 0 )
  {
    direction = 1;
  }
}


void figarc::write(std::ostream& stream) const
{
  stream <<"# "<<KOM<<"\n";	//kommentar
  stream <<TYP<<" ";	//(object_code)art des objekts?
  stream <<JUST<<" ";	//(sub_type)
  stream <<LINE_STYLE<<" ";	//(line_style)
  stream <<WIDTH<<" ";	//(thickness)
  stream <<PEN_COLOR<<" ";	//(pen_color)  [0,31]
  stream <<FUELL_COLOR<<" ";//(fill_color) [0,31]
  stream <<EBENE<<" ";	//(depth)Ebene
  stream <<PINSL<<" ";	//(pen_style)
  stream <<FILL<<" ";		//(area_fill)
  stream <<STYLE_VAL<<" ";	//(style_val)
  stream <<CAP_STYLE<<" ";	//(cap_style) Ecken 
	stream <<direction<<" ";	//(0: clockwise, 1: counterclockwise)
  stream <<ARROW_F<<" ";	//(forward_arrow) Vorwaerts Pfeil
  stream <<ARROW_B<<" ";	//(backward_arrow) Rueckwaertspfeil
	stream <<float(450*center_x)<<" ";	//(center of the arc)
	stream <<-float(450*center_y)<<" ";	//(center of the arc)
  for (int i=0; i<3; i++)
    stream <<int(floor(450*POSX[i]+0.5))<<" "<<int(-floor(450*POSY[i]+0.5))<<" ";
}


void figarc::translation(const double tx, const double ty)
{
  unsigned size = POSX.size();
  for ( unsigned i=0; i<size; ++i)
  {
    POSX[i] += tx;
    POSY[i] += ty;
  }
  center_x += tx;
  center_y += ty;
}

void figarc::rotation(const double degree)
{
  double pi = acos(-1);
  double cosmult = cos(pi*degree/180);
  double sinmult = sin(pi*degree/180);
  unsigned size = POSX.size();
  for ( unsigned i=0; i<size; ++i)
  {
    double x = POSX[i];
    double y = POSY[i];
    POSX[i] = cosmult*x - sinmult*y;
    POSY[i] = cosmult*y + sinmult*x;
  }  
  double x = center_x;
  double y = center_y;
  center_x = cosmult*x - sinmult*y;
  center_y = cosmult*y + sinmult*x;
}


void figarc::scaling(const double scale)
{
  unsigned size = POSX.size();
  for ( unsigned i=0; i<size; ++i)
  {
    POSX[i] *= scale;
    POSY[i] *= scale;
  }  
  center_x *= scale;
  center_y *= scale;
}


