#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE example
#include <boost/test/unit_test.hpp>
#include "figline.h"
#include "figstring.h"
#include "figarc.h"
#include "figcircle.h"


using namespace boost::unit_test;


class figlineTestproxy
{
public:
  typedef figline::koordarray koordarray;
  figlineTestproxy(figline const& line) : m_line(line) {}
  void GetPositions(koordarray& px, koordarray& py)
  {
    px = m_line.POSX;
    py = m_line.POSY;
  }
private:
  figline const& m_line;
};



BOOST_AUTO_TEST_CASE( figline_translation )
{
  figline linie(1,0,0,1);
  figlineTestproxy linieProxy(linie);
  figlineTestproxy::koordarray px,py;
  linie.translation(2,3);
  linieProxy.GetPositions(px,py);
  BOOST_CHECK_EQUAL(2.0,px.size());
  BOOST_CHECK_EQUAL(2.0,py.size());
  BOOST_CHECK_CLOSE(px[0],3.0,0.0000001);
  BOOST_CHECK_CLOSE(py[0],3.0,0.0000001);
  BOOST_CHECK_CLOSE(px[1],2.0,0.0000001);
  BOOST_CHECK_CLOSE(py[1],4.0,0.0000001);
}


BOOST_AUTO_TEST_CASE( figline_Rotation )
{
  figline linie(1,0,0,1);
  figlineTestproxy linieProxy(linie);
  figlineTestproxy::koordarray px,py;
  linie.rotation(90);
  linieProxy.GetPositions(px,py);
  BOOST_CHECK_EQUAL(2.0,px.size());
  BOOST_CHECK_EQUAL(2.0,py.size());
  BOOST_CHECK_CLOSE(px[0]+1,1.0,0.00000001);
  BOOST_CHECK_CLOSE(py[0],1.0,0.00001);
  BOOST_CHECK_CLOSE(px[1],-1.0,0.0000001);
  BOOST_CHECK_CLOSE(py[1]+1,1.0,0.0000001);
}


BOOST_AUTO_TEST_CASE( figline_skaling )
{
  figline linie(1.5,0,0,1.5);
  figlineTestproxy linieProxy(linie);
  figlineTestproxy::koordarray px,py;
  linie.scaling(0.6666666666666);
  linieProxy.GetPositions(px,py);
  BOOST_CHECK_EQUAL(2.0,px.size());
  BOOST_CHECK_EQUAL(2.0,py.size());
  BOOST_CHECK_CLOSE(px[0],1.0,0.0000001);
  BOOST_CHECK_CLOSE(py[0]+1,1.0,0.0000001);
  BOOST_CHECK_CLOSE(px[1]+1,1.0,0.0000001);
  BOOST_CHECK_CLOSE(py[1],1.0,0.0000001);
}


class figarcTestproxy
{
  typedef figarc::koordarray koordarray;
public:
  figarcTestproxy(figarc const& arc) : m_arc(arc) {}
  void GetPositions(double& cx, double& cy, koordarray& px, koordarray& py)
  {
    cx = m_arc.center_x;
    cy = m_arc.center_y;
    px = m_arc.POSX;
    py = m_arc.POSY;
  }
private:
  figarc const& m_arc;
};


BOOST_AUTO_TEST_CASE( figarc_translation )
{
  figarc bogen(1,0,0,1,-1,0);
  figarcTestproxy bogenproxy(bogen);
  double cx=0,cy=0;
  figarc::koordarray px;
  figarc::koordarray py;
  bogen.translation(2,3);
  bogenproxy.GetPositions(cx,cy,px,py);
  BOOST_CHECK_EQUAL(3,px.size());
  BOOST_CHECK_EQUAL(3,py.size());
  BOOST_CHECK_CLOSE(cx,2.0,0.0000001);
  BOOST_CHECK_CLOSE(cy,3.0,0.0000001);
  BOOST_CHECK_CLOSE(px[0],3.0,0.0000001);
  BOOST_CHECK_CLOSE(py[0],3.0,0.0000001);
  BOOST_CHECK_CLOSE(px[1],2.0,0.0000001);
  BOOST_CHECK_CLOSE(py[1],4.0,0.0000001);
  BOOST_CHECK_CLOSE(px[2],1.0,0.0000001);
  BOOST_CHECK_CLOSE(py[2],3.0,0.0000001);
}


BOOST_AUTO_TEST_CASE( figarc_Rotation )
{
  figarc bogen(3,2,2,3,1,2);
  figarcTestproxy bogenproxy(bogen);
  double cx=0,cy=0;
  figarc::koordarray px,py;
  bogen.rotation(90);
  bogenproxy.GetPositions(cx,cy,px,py);
  BOOST_CHECK_EQUAL(3,px.size());
  BOOST_CHECK_EQUAL(3,py.size());
  BOOST_CHECK_CLOSE(cx,-2.0,0.0000001);
  BOOST_CHECK_CLOSE(cy,2.0,0.0000001);
  BOOST_CHECK_CLOSE(px[0],-2.0,0.0000001);
  BOOST_CHECK_CLOSE(py[0],3.0,0.0000001);
  BOOST_CHECK_CLOSE(px[1],-3.0,0.0000001);
  BOOST_CHECK_CLOSE(py[1],2.0,0.0000001);
  BOOST_CHECK_CLOSE(px[2],-2.0,0.0000001);
  BOOST_CHECK_CLOSE(py[2],1.0,0.0000001);
}


BOOST_AUTO_TEST_CASE( figarc_scaling )
{
  figarc bogen(3,2,2,3,1,2);
  figarcTestproxy bogenproxy(bogen);
  double cx=0,cy=0;
  figarc::koordarray px,py;
  bogen.scaling(1.5);
  bogenproxy.GetPositions(cx,cy,px,py);
  BOOST_CHECK_EQUAL(3,px.size());
  BOOST_CHECK_EQUAL(3,py.size());
  BOOST_CHECK_CLOSE(cx,3.0,0.0000001);
  BOOST_CHECK_CLOSE(cy,3.0,0.0000001);
  BOOST_CHECK_CLOSE(px[0],4.5,0.0000001);
  BOOST_CHECK_CLOSE(py[0],3.0,0.0000001);
  BOOST_CHECK_CLOSE(px[1],3.0,0.0000001);
  BOOST_CHECK_CLOSE(py[1],4.5,0.0000001);
  BOOST_CHECK_CLOSE(px[2],1.5,0.0000001);
  BOOST_CHECK_CLOSE(py[2],3.0,0.0000001);
}


class figstringTestproxy
{
  typedef figstring::koordarray koordarray;
public:
  figstringTestproxy(figstring const& string) : m_string(string) {}
  void GetPositions(koordarray& px, koordarray& py)
  {
    px = m_string.POSX;
    py = m_string.POSY;
  }
private:
  figstring const& m_string;
};


BOOST_AUTO_TEST_CASE( figstring_Rotation )
{
  figstring text("Hello World",3,4);
  figstringTestproxy textProxy(text);
  figstring::koordarray px,py;
  text.rotation(270);
  textProxy.GetPositions(px,py);
  BOOST_CHECK_EQUAL(1,px.size());
  BOOST_CHECK_EQUAL(1,py.size());
  BOOST_CHECK_CLOSE(px[0],4.0,0.0000001);
  BOOST_CHECK_CLOSE(py[0],-3.0,0.0000001);
}

BOOST_AUTO_TEST_CASE( figstring_translation )
{
  figstring text("Hello World",3,4);
  figstringTestproxy textProxy(text);
  figstring::koordarray px,py;
  text.translation(-3,2);
  textProxy.GetPositions(px,py);
  BOOST_CHECK_EQUAL(1,px.size());
  BOOST_CHECK_EQUAL(1,py.size());
  BOOST_CHECK_CLOSE(px[0]+1, 1.0, 0.0000001);
  BOOST_CHECK_CLOSE(py[0], 6.0, 0.0000001);
}

BOOST_AUTO_TEST_CASE( figstring_scaling )
{
  figstring text("Hello World",3,4);
  figstringTestproxy textProxy(text);
  figstring::koordarray px,py;
  text.scaling(0.5);
  textProxy.GetPositions(px,py);
  BOOST_CHECK_EQUAL(1,px.size());
  BOOST_CHECK_EQUAL(1,py.size());
  BOOST_CHECK_CLOSE(px[0],1.5,0.0000001);
  BOOST_CHECK_CLOSE(py[0],2.0,0.0000001);
}

