#include "figline.h"
#include <ostream>
#include <cmath>

using std::floor;
using std::ostream;



figline::figline(const double x1,const double y1,const double x2,const double y2) 
{
  KOM   = "(POLYLINE)";	//kommentar
  TYP   =    2;		//(object_code) objekt Typ
  JUST  =    1;		//(sub_type)textausrichtung 0=links,1=mitte,2=rechts
  LINE_STYLE=0;		//(line_style)
  WIDTH =    1;		//(thickness) breite
  PEN_COLOR= 0;		//(pen_color)farbe:0=schwarz;1=blau;2=gruen;3=hellblau,4=rot,5=lila,6=gelb [0,31]
  FUELL_COLOR=7;		//(fill_color)farbe:0=schwarz;1=blau;2=gruen;3=hellblau,4=rot,5=lila,6=gelb [0,31]
  EBENE    =50;		//(depth)Ebene
  PINSL =   -1;		//(pen_style)
  FILL  =   -1;		//(area_fill)
  STYLE_VAL=.0;		//(style_val)
  JOIN_STYLE=0;		//(join_style)
  CAP_STYLE =0;		//(cap_style) Ecken 
  RADIUS   =-1;		//(radius)1/80 inch
  ARROW_F  = 0;		//(forward_arrow) Vorwaerts Pfeil 0=off 1=on
  ARROW_B  = 0;		//(backward_arrow) Rueckwaertspfeil 0=off 1=on
  POSX.resize(2);
  POSY.resize(2);
  POSX[0] = x1;
  POSY[0] = y1;
  POSX[1] = x2;
  POSY[1] = y2;
}


void figline::write(ostream& stream) const
{  
  stream <<"# "<<KOM<<"\n";	//kommentar
  stream <<TYP<<" ";	//(object_code)art des objekts?
  stream <<JUST<<" ";	//(sub_type)
  stream <<LINE_STYLE<<" ";	//(line_style)
  stream <<WIDTH<<" ";	//(thickness)
  stream <<PEN_COLOR<<" ";	//(pen_color)  [0,31]
  stream <<FUELL_COLOR<<" ";//(fill_color) [0,31]
  stream <<EBENE<<" ";	//(depth)Ebene
  stream <<PINSL<<" ";	//(pen_style)
  stream <<FILL<<" ";		//(area_fill)
  stream <<STYLE_VAL<<" ";	//(style_val)
  stream <<JOIN_STYLE<<" ";	//(join_style)
  stream <<CAP_STYLE<<" ";	//(cap_style) Ecken 
  stream <<RADIUS<<" ";	//(radius)
  stream <<ARROW_F<<" ";	//(forward_arrow) Vorwaerts Pfeil
  stream <<ARROW_B<<" ";	//(backward_arrow) Rueckwaertspfeil
  int size = POSX.size();
  stream <<size<<"\n\t ";	//(npoints) Anzahl der Punkte
  for (int i=0;i<size;i++)
    stream <<int(floor(POSX[i]*450+0.5))<<" "<<int(-floor(POSY[i]*450+0.5))<<" ";
}


void figline::translation(const double zx, const double zy)
{
  unsigned size = POSX.size();
  for ( unsigned i=0; i<size; ++i)
  {
    POSX[i] = POSX[i] + zx;
    POSY[i] = POSY[i] + zy;
  }
}


void figline::rotation(const double degree)
{
  double pi = acos(-1);
  double cosmult = cos(pi*degree/180.0);
  double sinmult = sin(pi*degree/180.0);
  unsigned size = POSX.size();
  for ( unsigned i=0; i<size; ++i)
  {
    double x = POSX[i];
    double y = POSY[i];
    POSX[i] = cosmult*x - sinmult*y;
    POSY[i] = cosmult*y + sinmult*x;
  }  
}


void figline::scaling(const double scale)
{
  unsigned size = POSX.size();
  for ( unsigned i=0; i<size; ++i)
  {
    POSX[i] *= scale;
    POSY[i] *= scale;
  }  
}




