#ifndef FIGOBJECT
#define FIGOBJECT

#include "figelement.h"
#include <memory>
#include <string>
#include <iostream>


using std::auto_ptr;



class figobject : public figelement
{
  public:
    friend class figobjectTestproxy;
  
    template <class T>
    inline figobject(T const& t);
    figobject(figobject const& fc);
    figobject& operator=(figobject const& fc);

    void write(ostream& stream) const;
    void translation(const double zx, const double zy);
    void rotation(const double degree);
    void scaling(const double scale);

  private:
    struct impl 
    {
      virtual ~impl() {}
      virtual impl* clone() const = 0; 
      virtual void write(ostream& stream) const = 0;
      virtual void translation(const double zx, const double zy) = 0;
      virtual void rotation(const double degree) = 0;
      virtual void scaling(const double scale) = 0;    
    };

    template <class T>
    struct wrapper : public impl
    { 
        inline wrapper(T const& object);
        inline impl* clone() const;
        inline void write(ostream& stream) const;
        inline void translation(const double zx, const double zy);
        inline void rotation(const double degree);
        inline void scaling(const double scale);
      private:
        T m_object;
    };

    auto_ptr<impl> pimpl;
};




template <class T>
figobject::figobject(T const& t) : pimpl(new wrapper<T>(t)) 
{
}

template <class T>
figobject::wrapper<T>::wrapper(T const& object)
 : m_object(object) 
{
}

template <class T>
figobject::impl* figobject::wrapper<T>::clone() const 
{ 
  return new wrapper<T>(this->m_object); 
}

template <class T>
void figobject::wrapper<T>::write(ostream& stream) const 
{ 
  m_object.write(stream); 
}

template <class T>
void figobject::wrapper<T>::translation(const double zx, const double zy) 
{ 
  m_object.translation(zx,zy); 
}

template <class T>
void figobject::wrapper<T>::rotation(const double degree)
{ 
  m_object.rotation(degree); 
}

template <class T>
void figobject::wrapper<T>::scaling(const double scale)
{ 
  m_object.scaling(scale); 
}



#endif //FIGOBJECT
