#include "figstring.h"
#include <ostream>
#include <cmath>

using std::floor;
using std::ostream;
using std::acos;
using std::cos;
using std::sin;



figstring::figstring(const string text,const double x,const double y)
{
  KOM   = "(TEXT)";		//kommentar
  TYP   =  4;		//(object_code) objekt Typ
  JUST  =  1;		//(sub_type)textausrichtung 0=links,1=mitte,2=rechts
  COLOR =  0;		//(color)farbe:0=schwarz;1=blau;2=gruen;3=hellblau,4=rot,5=lila,6=gelb [0,31]
  EBENE =   49;		//(depth)Ebene
  PINSL =   -1;		//(pen_style)
  FONT  =  0;		//(font)
  SIZE  =  12;		//(font_size)Size
  RAD   =   .0;		//(angle)winkel (rad)
  FLAG  =  6;		//(font_flags)Flags: 1.bit=rigid,2.bit=spezial,3.bit=PS/latex,4.bit=hidden
  PS1   =  450;		//(height)schriftart
  PS2   = 3675;		//(length)schriftart
  POSX.resize(1);
  POSY.resize(1);
  TEXT = text;
  POSX[0] = x;
  POSY[0] = y;
}



void figstring::write(ostream& stream) const
{
  stream <<"# "<<KOM<<"\n";	//kommentar
  stream <<TYP<<" ";	//(object_code)art des objekts?
  stream <<JUST<<" ";	//(sub_type)textausrichtung(Justifikation) //0=links,1=mitte,2=rechts
  stream <<COLOR<<" ";	//(color)farbe:0=schwarz;1=blau;2=gruen;3=hellblau,4=rot,5=lila,6=gelb [0,31]
  stream <<EBENE<<" ";	//(depth)Ebene
  stream <<PINSL<<" ";	//(pen_style)
  stream <<FONT<<" ";	//(font)
  stream <<SIZE<<" ";	//(font_size)Size
  stream <<RAD<<" ";	//(angle)winkel (rad)
  stream <<FLAG<<" ";	//(font_flags)Flags: 1.bit=rigid,2.bit=spezial,4.bit=hidden,3.bit=PS/latex font?
  stream <<PS1<<" ";	//(height)schriftart
  stream <<PS2<<" ";	//(length)schriftart
  stream <<int(floor(POSX[0]*450+0.5))<<" ";	//(x)pos y
  stream <<int(-floor(POSY[0]*450+0.5))<<" ";	//(y)pos x (negativ)
  for (unsigned int i=0;i<TEXT.length();i++)
  {
    if (TEXT[i]>127)
      stream <<"\\"<<((TEXT[i])&7)<<(((TEXT[i])&56)>>3)<<(((TEXT[i])&448)<<6);
    else if (TEXT[i]==92)
      stream <<"\\\\";
    else
      stream <<TEXT[i];
  }
  stream <<"\\001\n";
}


void figstring::translation(const double zx, const double zy)
{
  unsigned size = POSX.size();
  for ( unsigned i=0; i<size; ++i)
  {
    POSX[i] += zx;
    POSY[i] += zy;
  }
}


void figstring::rotation(const double degree)
{
  double pi = acos(-1);
  double cosmult = cos(pi*degree/180);
  double sinmult = sin(pi*degree/180);
  unsigned size = POSX.size();
  for ( unsigned i=0; i<size; ++i)
  {
    double x = POSX[i];
    double y = POSY[i];
    POSX[i] = cosmult*x - sinmult*y;
    POSY[i] = cosmult*y + sinmult*x;
  }  
}


void figstring::scaling(const double scale)
{
  unsigned size = POSX.size();
  for ( unsigned i=0; i<size; ++i)
  {
    POSX[i] *= scale;
    POSY[i] *= scale;
  }  
}


